﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel

Partial Public Class MainWindow
  Public WithEvents Data As CustomerList = CustomerList.GetList()
  Private WithEvents cust As Customer

#Region "UI Event Handlers"

  Private Sub MainWindow_Loaded( _
    ByVal sender As System.Object, _
    ByVal e As RoutedEventArgs) _
    Handles MainWindow.Loaded

    ' Hook up event handlers and display the
    ' correct stuff in the various controls:
    SetupData()
  End Sub

  Private Sub ResetButton_Click( _
    ByVal sender As System.Object, _
    ByVal e As RoutedEventArgs) _
    Handles ResetButton.Click

    ' Call the Reset method in the 
    ' CustomerList class, and then 
    ' set up the data again (that is, 
    ' hook up event handlers and display the
    ' correct stuff in the various controls):
    CustomerList.Reset()
    SetupData()
  End Sub

  Private Sub NewItemTextBox_TextChanged( _
    ByVal sender As System.Object, _
    ByVal e As TextChangedEventArgs) _
    Handles NewItemTextBox.TextChanged

    ' Only enable the New Item button
    ' if there's something in the 
    ' new item text box:
    NewItemButton.IsEnabled = _
      (NewItemTextBox.Text.Length > 0)
  End Sub

  Private Sub DeleteItemButton_Click( _
    ByVal sender As System.Object, _
    ByVal e As System.Windows.RoutedEventArgs) _
    Handles DeleteItemButton.Click

    Dim index As Integer = ItemComboBox.SelectedIndex
    If index >= 0 Then
      RemoveHandler Data.Item(index).PropertyChanged, _
        AddressOf HandlePropertyChanged

      Data.RemoveAt(index)

      ' Anything left in the list? If so
      ' reset the list box. Otherwise, 
      ' clear out the text box:
      If Data.Count > 0 Then
        ResetListBox()
      Else
        ChangeItemTextBox.Clear()
      End If
    End If
  End Sub

  Private Sub NewItemButton_Click( _
    ByVal sender As System.Object, _
    ByVal e As RoutedEventArgs) _
    Handles NewItemButton.Click

    ' Add the new item to the list of customers, 
    ' hook up the event handler, and 
    ' clear out the text box:
    cust = New Customer(NewItemTextBox.Text)
    HookupChangeEventHandler(cust)
    Data.Add(cust)

    ' Leave the text box empty for the
    ' next addition.
    NewItemTextBox.Clear()
  End Sub

  Private Sub ChangeItemButton_Click( _
    ByVal sender As System.Object, _
    ByVal e As RoutedEventArgs) _
    Handles ChangeItemButton.Click

    ' Something selected? If so, change the 
    ' Name property of the current data item. 
    ' This should raise the PropertyChanged event 
    ' for the item, as well:
    Dim index As Integer = ItemComboBox.SelectedIndex
    If index >= 0 Then
      Data.Item(index).Name = ChangeItemTextBox.Text
    End If
  End Sub

  Private Sub ItemComboBox_SelectionChanged( _
    ByVal sender As System.Object, _
    ByVal e As SelectionChangedEventArgs) _
    Handles ItemComboBox.SelectionChanged

    ' Something selected? If so, select the corresponding
    ' item in the ListBox, and set the text 
    ' in the text box:
    Dim index As Integer = ItemComboBox.SelectedIndex
    If index >= 0 Then
      ItemListBox.SelectedIndex = index
      ChangeItemTextBox.Text = Data.Item(index).Name
    End If
  End Sub

  Private Sub ItemListBox_SelectionChanged( _
    ByVal sender As System.Object, _
    ByVal e As SelectionChangedEventArgs) _
    Handles ItemListBox.SelectionChanged

    Dim index As Integer = ItemListBox.SelectedIndex
    ' Anything selected?
    If index >= 0 Then
      ' Is the index different than the last time
      ' something was selected?
      If index <> ItemComboBox.SelectedIndex Then
        ' This triggers the ItemComboBox_SelectionChanged
        ' event handler, which resets the text 
        ' in the text box:
        ItemComboBox.SelectedIndex = index
      Else
        ' If the index didn't change, you must
        ' manually set the text box's text:
        ChangeItemTextBox.Text = Data.Item(index).Name
      End If
    End If
  End Sub

#End Region

#Region "Change Event Handlers"

  Private Sub HandlePropertyChanged( _
  ByVal sender As Object, _
  ByVal e As PropertyChangedEventArgs)

    ' In this particular application, you only
    ' want to bother with this code for the 
    ' first window, although both will run the code.
    ' In this case, if the event was raised by
    ' the window whose title is "Observable Collection 1"
    ' then process the event:
    If Me.Title.EndsWith("1") Then
      Dim propName As String = e.PropertyName
      Dim myCustomer As Customer = CType(sender, Customer)

      ' Unfortunately, no one hands you the 
      ' old property value, or the new property value. 
      ' You can use Reflection to retrieve the 
      ' new property value, given the object that
      ' raised the event and the name of the property:
      Dim propInfo As System.Reflection.PropertyInfo = _
        GetType(Customer).GetProperty(propName)
      Dim value As Object = _
        propInfo.GetValue(myCustomer, Nothing)

      MessageBox.Show(String.Format( _
        "You changed the property '{0}' to '{1}'", _
        propName, value))
    End If
  End Sub

  Private Sub Data_CollectionChanged( _
    ByVal sender As Object, _
    ByVal e As NotifyCollectionChangedEventArgs) _
    Handles Data.CollectionChanged

    ' Because the collection raises this event, 
    ' you can modify your user interface on any window
    ' that displays controls bound to the data. On both
    ' windows, if you add or remove an item, all the 
    ' controls update to indicate the new collection!

    ' Did you add or remove an item in the collection?
    If e.Action = NotifyCollectionChangedAction.Add Or _
      e.Action = NotifyCollectionChangedAction.Remove Then

      ' Set the list of integers in the combo box:
      SetComboDataSource()

      ' Enable buttons as necessary:
      EnableButtons()
    End If
  End Sub

#End Region

#Region "Support Procedures"

  Private Sub HookupChangeEventHandlers()
    ' Hook up change notification. This really
    ' isn't part of the ObservableCollection class,
    ' but it seems that you would want to know
    ' how to track modifications in the 
    ' properties of the children of the collection:
    For Each cust As Customer In Data
      HookupChangeEventHandler(cust)
    Next
  End Sub

  Private Sub HookupChangeEventHandler(ByVal cust As Customer)
    ' Add a PropertyChanged event handler for 
    ' the specified Customer instance:
    AddHandler cust.PropertyChanged, _
      AddressOf HandlePropertyChanged
  End Sub

  Private Sub ResetListBox()
    ' Reset the list box, setting 
    ' the selected index to 0 and 
    ' moving focus to the list box:
    ItemListBox.SelectedIndex = 0
    ItemListBox.Focus()
  End Sub

  Private Sub SetupData()
    ' At startup, or when you reset the 
    ' data, you need to hookup the 
    ' PropertyChanged event handlers.
    HookupChangeEventHandlers()

    If Data.Count > 0 Then
      ' Set up the data source for the 
      ' ComboBox and fix up the ListBox:
      SetComboDataSource()
      ResetListBox()
    End If
  End Sub

  Private Sub SetComboDataSource()
    ' Set the list of integers shown in the 
    ' combo box:
    ItemComboBox.ItemsSource = _
      Enumerable.Range(0, Data.Count)
  End Sub

  Private Sub EnableButtons()
    ' Yes, you could accomplish this same thing
    ' using declarative binding, but I wanted
    ' to keep this simple...
    Dim enabled As Boolean = Data.Count > 0
    ChangeItemButton.IsEnabled = enabled
    DeleteItemButton.IsEnabled = enabled
    ChangeItemTextBox.IsEnabled = enabled
    ItemComboBox.IsEnabled = enabled
    ItemListBox.IsEnabled = enabled
  End Sub

#End Region

End Class
